@@ -11,6 +11,7 @@
 
 #include <functional>
 #include <numeric>
+#include <stdexcept>
 
 #include "xiterable.hpp"
 #include "xiterator.hpp"
@@ -21,17 +22,6 @@
 
 namespace xt
 {
-
-    namespace check_policy
-    {
-        struct none
-        {
-        };
-        struct full
-        {
-        };
-    }
-
     template <class D>
     struct xcontainer_iterable_types
     {
@@ -39,10 +29,10 @@ namespace xt
         using container_type = typename xcontainer_inner_types<D>::container_type;
         using iterator = typename container_type::iterator;
         using const_iterator = typename container_type::const_iterator;
+        using reverse_iterator = typename container_type::reverse_iterator;
+        using const_reverse_iterator = typename container_type::const_reverse_iterator;
         using stepper = xstepper<D>;
         using const_stepper = xstepper<const D>;
-        using broadcast_iterator = xiterator<stepper, inner_shape_type*>;
-        using const_broadcast_iterator = xiterator<const_stepper, inner_shape_type*>;
     };
 
     /**
@@ -90,8 +80,8 @@ namespace xt
         using stepper = typename iterable_base::stepper;
         using const_stepper = typename iterable_base::const_stepper;
 
-        using broadcast_iterator = typename iterable_base::broadcast_iterator;
-        using const_broadcast_iterator = typename iterable_base::broadcast_iterator;
+        using reverse_iterator = typename iterable_base::reverse_iterator;
+        using const_reverse_iterator = typename iterable_base::const_reverse_iterator;
 
         size_type size() const noexcept;
 
@@ -101,19 +91,6 @@ namespace xt
         const inner_strides_type& strides() const noexcept;
         const inner_backstrides_type& backstrides() const noexcept;
 
-        void transpose();
-
-        template <class S, class Tag = check_policy::none>
-        void transpose(S&& permutation, Tag check_policy = Tag());
-
-#ifdef X_OLD_CLANG
-        template <class I, class Tag = check_policy::none>
-        void transpose(std::initializer_list<I> permutation, Tag check_policy = Tag());
-#else
-        template <class I, std::size_t N, class Tag = check_policy::none>
-        void transpose(const I (&permutation)[N], Tag check_policy = Tag());
-#endif
-
         template <class... Args>
         reference operator()(Args... args);
 
@@ -151,6 +128,14 @@ namespace xt
         const_iterator cbegin() const noexcept;
         const_iterator cend() const noexcept;
 
+        reverse_iterator rbegin() noexcept;
+        reverse_iterator rend() noexcept;
+
+        const_reverse_iterator rbegin() const noexcept;
+        const_reverse_iterator rend() const noexcept;
+        const_reverse_iterator crbegin() const noexcept;
+        const_reverse_iterator crend() const noexcept;
+
         template <class S>
         stepper stepper_begin(const S& shape) noexcept;
         template <class S>
@@ -161,6 +146,9 @@ namespace xt
         template <class S>
         const_stepper stepper_end(const S& shape) const noexcept;
 
+        iterator data_xend() noexcept;
+        const_iterator data_xend() const noexcept;
+
     protected:
 
         xcontainer() = default;
@@ -174,11 +162,8 @@ namespace xt
 
     private:
 
-        template <class S>
-        void transpose_impl(S&& permutation, check_policy::none);
-
-        template <class S>
-        void transpose_impl(S&& permutation, check_policy::full);
+        template <class It>
+        It data_xend_impl(It end) const noexcept;
 
         inner_shape_type& mutable_shape();
         inner_strides_type& mutable_strides();
@@ -197,10 +182,11 @@ namespace xt
      * not embed the data container, this responsibility is delegated to the inheriting
      * classes.
      *
-     * @tparam D The derived type, i.e. the inheriting class for which xstrided
+     * @tparam D The derived type, i.e. the inheriting class for which xstrided_container
      *           provides the partial imlpementation of xcontainer.
+     * @tparam L The layout_type of the xstrided_container.
      */
-    template <class D>
+    template <class D, layout_type L>
     class xstrided_container : public xcontainer<D>
     {
 
@@ -220,9 +206,17 @@ namespace xt
         using inner_strides_type = typename base_type::inner_strides_type;
         using inner_backstrides_type = typename base_type::inner_backstrides_type;
 
-        void reshape(const shape_type& shape);
-        void reshape(const shape_type& shape, layout l);
-        void reshape(const shape_type& shape, const strides_type& strides);
+        static constexpr layout_type static_layout = L;
+        static constexpr bool contiguous_layout = static_layout != layout_type::dynamic;
+
+        template <class S = shape_type>
+        void reshape(const S& shape, bool force = false);
+        template <class S = shape_type>
+        void reshape(const S& shape, layout_type l);
+        template <class S = shape_type>
+        void reshape(const S& shape, const strides_type& strides);
+
+        layout_type layout() const noexcept;
 
     protected:
 
@@ -251,12 +245,20 @@ namespace xt
         inner_shape_type m_shape;
         inner_strides_type m_strides;
         inner_backstrides_type m_backstrides;
+        layout_type m_layout = L;
     };
 
     /******************************
      * xcontainer implementation *
      ******************************/
 
+    template <class D>
+    template <class It>
+    inline It xcontainer<D>::data_xend_impl(It end) const noexcept
+    {
+        return end - 1 + strides().back();
+    }
+
     template <class D>
     inline auto xcontainer<D>::mutable_shape() -> inner_shape_type&
     {
@@ -335,100 +337,6 @@ namespace xt
     {
         return derived_cast().backstrides_impl();
     }
-
-    /**
-     * Transposes the container inplace by reversing the dimensions.
-     */
-    template <class D>
-    inline void xcontainer<D>::transpose()
-    {
-        // reverse stride and shape
-        std::reverse(mutable_shape().begin(), mutable_shape().end());
-        std::reverse(mutable_strides().begin(), mutable_strides().end());
-        std::reverse(mutable_backstrides().begin(), mutable_backstrides().end());
-    }
-
-    /**
-     * Transposes the container inplace by permuting the shape with @p permutation.
-     * @param permutation the sequence containing permutation
-     * @param check_policy the check level (check_policy::full() or check_policy::none())
-     * @tparam Tag selects the level of error checking on permutation vector defaults to check_policy::none.
-     */
-    template <class D>
-    template <class S, class Tag>
-    inline void xcontainer<D>::transpose(S&& permutation, Tag check_policy)
-    {
-        transpose_impl(std::forward<S>(permutation), check_policy);
-    }
-
-#ifdef X_OLD_CLANG
-    template <class D>
-    template <class I, class Tag>
-    inline void xcontainer<D>::transpose(std::initializer_list<I> permutation, Tag check_policy)
-    {
-        std::vector<I> perm(permutation);
-        transpose_impl(std::move(perm), check_policy);
-    }
-#else
-    template <class D>
-    template <class I, std::size_t N, class Tag>
-    inline void xcontainer<D>::transpose(const I (&permutation)[N], Tag check_policy)
-    {
-        transpose_impl(permutation, check_policy);
-    }
-#endif
-
-    template <class D>
-    template <class S>
-    inline void xcontainer<D>::transpose_impl(S&& permutation, check_policy::full)
-    {
-        // check if axis appears twice in permutation
-        for (size_type i = 0; i < container_size(permutation); ++i)
-        {
-            for (size_type j = i + 1; j < container_size(permutation); ++j)
-            {
-                if (permutation[i] == permutation[j])
-                {
-                    throw transpose_error("Permutation contains axis more than once");
-                }
-            }
-        }
-        transpose_impl(permutation, check_policy::none());
-    }
-
-    template <class D>
-    template <class S>
-    inline void xcontainer<D>::transpose_impl(S&& permutation, check_policy::none)
-    {
-        if (container_size(permutation) != dimension())
-        {
-            throw transpose_error("Permutation does not have the same size as shape");
-        }
-
-        // permute stride and shape
-        strides_type temp_strides;
-        resize_container(temp_strides, strides().size());
-
-        shape_type temp_shape;
-        resize_container(temp_shape, shape().size());
-
-        shape_type temp_backstrides;
-        resize_container(temp_backstrides, backstrides().size());
-
-        for (size_type i = 0; i < shape().size(); ++i)
-        {
-            if (size_type(permutation[i]) >= dimension())
-            {
-                throw transpose_error("Permutation contains wrong axis");
-            }
-            temp_shape[i] = shape()[permutation[i]];
-            temp_strides[i] = strides()[permutation[i]];
-            temp_backstrides[i] = backstrides()[permutation[i]];
-        }
-        mutable_shape() = std::move(temp_shape);
-        mutable_strides() = std::move(temp_strides);
-        mutable_backstrides() = std::move(temp_backstrides);
-    }
     //@}
 
 
@@ -675,6 +583,71 @@ namespace xt
     }
     //@}
 
+    /**
+     * @name Reverse iterators
+     */
+    //@{
+    /**
+     * Returns an iterator to the first element of the reversed buffer containing
+     * the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::rbegin() noexcept -> reverse_iterator
+    {
+        return data().rbegin();
+    }
+
+    /**
+     * Returns an iterator to the element following the last element of
+     * the reversed buffer containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::rend() noexcept -> reverse_iterator
+    {
+        return data().rend();
+    }
+
+    /**
+     * Returns a constant iterator to the first element of the reversed
+     * buffer containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::rbegin() const noexcept -> const_reverse_iterator
+    {
+        return data().rbegin();
+    }
+
+    /**
+     * Returns a constant iterator to the element following the last
+     * element of the reversed buffer containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::rend() const noexcept -> const_reverse_iterator
+    {
+        return data().rend();
+    }
+
+    /**
+     * Returns a constant iterator to the first element of the reversed
+     * buffer containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::crbegin() const noexcept -> const_reverse_iterator
+    {
+        return data().crbegin();
+    }
+
+    /**
+     * Returns a constant iterator to the element following the last
+     * element of the reversed buffer containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::crend() const noexcept -> const_reverse_iterator
+    {
+        return data().crend();
+    }
+    //@}
+
     /***************
      * stepper api *
      ***************/
@@ -692,7 +665,7 @@ namespace xt
     inline auto xcontainer<D>::stepper_end(const S& shape) noexcept -> stepper
     {
         size_type offset = shape.size() - dimension();
-        return stepper(static_cast<derived_type*>(this), data().end(), offset);
+        return stepper(static_cast<derived_type*>(this), data_xend(), offset);
     }
 
     template <class D>
@@ -708,90 +681,124 @@ namespace xt
     inline auto xcontainer<D>::stepper_end(const S& shape) const noexcept -> const_stepper
     {
         size_type offset = shape.size() - dimension();
-        return const_stepper(static_cast<const derived_type*>(this), data().end(), offset);
+        return const_stepper(static_cast<const derived_type*>(this), data_xend(), offset);
+    }
+
+    template <class D>
+    inline auto xcontainer<D>::data_xend() noexcept -> iterator
+    {
+        return data_xend_impl(data().end());
+    }
+
+    template <class D>
+    inline auto xcontainer<D>::data_xend() const noexcept -> const_iterator
+    {
+        return data_xend_impl(data().end());
     }
 
     /*************************************
      * xstrided_container implementation *
      *************************************/
 
-    template <class D>
-    inline xstrided_container<D>::xstrided_container() noexcept
+    template <class D, layout_type L>
+    inline xstrided_container<D, L>::xstrided_container() noexcept
         : base_type()
     {
         m_shape = make_sequence<inner_shape_type>(base_type::dimension(), 1);
     }
 
-    template <class D>
-    inline xstrided_container<D>::xstrided_container(inner_shape_type&& shape, inner_strides_type&& strides) noexcept
+    template <class D, layout_type L>
+    inline xstrided_container<D, L>::xstrided_container(inner_shape_type&& shape, inner_strides_type&& strides) noexcept
         : base_type(), m_shape(std::move(shape)), m_strides(std::move(strides))
     {
         m_backstrides = make_sequence<inner_backstrides_type>(m_shape.size(), 0);
         adapt_strides(m_shape, m_strides, m_backstrides);
     }
 
-    template <class D>
-    inline auto xstrided_container<D>::shape_impl() noexcept -> inner_shape_type&
+    template <class D, layout_type L>
+    inline auto xstrided_container<D, L>::shape_impl() noexcept -> inner_shape_type&
     {
         return m_shape;
     }
 
-    template <class D>
-    inline auto xstrided_container<D>::shape_impl() const noexcept -> const inner_shape_type&
+    template <class D, layout_type L>
+    inline auto xstrided_container<D, L>::shape_impl() const noexcept -> const inner_shape_type&
     {
         return m_shape;
     }
 
-    template <class D>
-    inline auto xstrided_container<D>::strides_impl() noexcept -> inner_strides_type&
+    template <class D, layout_type L>
+    inline auto xstrided_container<D, L>::strides_impl() noexcept -> inner_strides_type&
     {
         return m_strides;
     }
 
-    template <class D>
-    inline auto xstrided_container<D>::strides_impl() const noexcept -> const inner_strides_type&
+    template <class D, layout_type L>
+    inline auto xstrided_container<D, L>::strides_impl() const noexcept -> const inner_strides_type&
     {
         return m_strides;
     }
 
-    template <class D>
-    inline auto xstrided_container<D>::backstrides_impl() noexcept -> inner_backstrides_type&
+    template <class D, layout_type L>
+    inline auto xstrided_container<D, L>::backstrides_impl() noexcept -> inner_backstrides_type&
     {
         return m_backstrides;
     }
 
-    template <class D>
-    inline auto xstrided_container<D>::backstrides_impl() const noexcept -> const inner_backstrides_type&
+    template <class D, layout_type L>
+    inline auto xstrided_container<D, L>::backstrides_impl() const noexcept -> const inner_backstrides_type&
     {
         return m_backstrides;
     }
 
+    /**
+     * Return the layout_type of the container
+     * @return layout_type of the container
+     */
+    template <class D, layout_type L>
+    layout_type xstrided_container<D, L>::layout() const noexcept
+    {
+        return m_layout;
+    }
+
     /**
      * Reshapes the container.
      * @param shape the new shape
+     * @param force force reshaping, even if the shape stays the same (default: false)
      */
-    template <class D>
-    inline void xstrided_container<D>::reshape(const shape_type& shape)
+    template <class D, layout_type L>
+    template <class S>
+    inline void xstrided_container<D, L>::reshape(const S& shape, bool force)
     {
-        if (shape != m_shape)
+        if (m_shape.size() != shape.size() || !std::equal(std::begin(shape), std::end(shape), std::begin(m_shape)) || force)
         {
-            reshape(shape, layout::row_major);
+            if (m_layout == layout_type::dynamic || m_layout == layout_type::any)
+            {
+                m_layout = layout_type::row_major;  // fall back to row major
+            }
+            m_shape = forward_sequence<shape_type>(shape);
+            resize_container(m_strides, m_shape.size());
+            resize_container(m_backstrides, m_shape.size());
+            size_type data_size = compute_strides(m_shape, m_layout, m_strides, m_backstrides);
+            this->data().resize(data_size);
         }
     }
 
     /**
      * Reshapes the container.
      * @param shape the new shape
-     * @param l the new layout
+     * @param l the new layout_type
      */
-    template <class D>
-    inline void xstrided_container<D>::reshape(const shape_type& shape, layout l)
+    template <class D, layout_type L>
+    template <class S>
+    inline void xstrided_container<D, L>::reshape(const S& shape, layout_type l)
     {
-        m_shape = shape;
-        resize_container(m_strides, m_shape.size());
-        resize_container(m_backstrides, m_shape.size());
-        size_type data_size = compute_strides(m_shape, l, m_strides, m_backstrides);
-        this->data().resize(data_size);
+        if (L != layout_type::dynamic && l != L)
+        {
+            throw std::runtime_error("Cannot change layout_type if template parameter not layout_type::dynamic.");
+        }
+        m_layout = l;
+        reshape(shape, true);
     }
 
     /**
@@ -799,13 +806,19 @@ namespace xt
      * @param shape the new shape
      * @param strides the new strides
      */
-    template <class D>
-    inline void xstrided_container<D>::reshape(const shape_type& shape, const strides_type& strides)
+    template <class D, layout_type L>
+    template <class S>
+    inline void xstrided_container<D, L>::reshape(const S& shape, const strides_type& strides)
     {
-        m_shape = shape;
+        if (L != layout_type::dynamic)
+        {
+            throw std::runtime_error("Cannot reshape with custom strides when layout() is != layout_type::dynamic.");
+        }
+        m_shape = forward_sequence<shape_type>(shape);
         m_strides = strides;
         resize_container(m_backstrides, m_strides.size());
         adapt_strides(m_shape, m_strides, m_backstrides);
+        m_layout = layout_type::dynamic;
         this->data().resize(compute_size(m_shape));
     }
 }
