@@ -71,10 +71,10 @@ namespace xt
         using inner_shape_type = typename xreducer_shape_type<typename xexpression_type::shape_type, X>::type;
         using const_stepper = xindexed_stepper<xreducer<F, CT, X>>;
         using stepper = const_stepper;
-        using const_broadcast_iterator = xiterator<const_stepper, inner_shape_type*>;
-        using broadcast_iterator = const_broadcast_iterator;
-        using const_iterator = const_broadcast_iterator;
+        using const_iterator = xiterator<const_stepper, inner_shape_type*, DEFAULT_LAYOUT>;
         using iterator = const_iterator;
+        using const_reverse_iterator = std::reverse_iterator<const_iterator>;
+        using reverse_iterator = std::reverse_iterator<iterator>;
     };
 
     /**
@@ -119,11 +119,8 @@ namespace xt
         using stepper = typename iterable_base::stepper;
         using const_stepper = typename iterable_base::const_stepper;
 
-        using broadcast_iterator = typename iterable_base::broadcast_iterator;
-        using const_broadcast_iterator = typename iterable_base::const_broadcast_iterator;
-
-        using iterator = typename iterable_base::iterator;
-        using const_iterator = typename iterable_base::const_iterator;
+        static constexpr layout_type static_layout = layout_type::dynamic;
+        static constexpr bool contiguous_layout = false;
 
         template <class Func, class CTA, class AX>
         xreducer(Func&& func, CTA&& e, AX&& axes);
@@ -131,6 +128,7 @@ namespace xt
         size_type size() const noexcept;
         size_type dimension() const noexcept;
         const inner_shape_type& shape() const noexcept;
+        layout_type layout() const noexcept;
 
         template <class... Args>
         const_reference operator()(Args... args) const;
@@ -466,6 +464,15 @@ namespace xt
     {
         return m_shape;
     }
+
+    /**
+     * Returns the shape of the expression.
+     */
+    template <class F, class CT, class X>
+    inline layout_type xreducer<F, CT, X>::layout() const noexcept
+    {
+        return static_layout;
+    }
     //@}
 
     /**
