@@ -42,6 +42,12 @@ namespace xt
     template <class... T>
     struct and_;
 
+    template <bool... B>
+    struct or_c;
+
+    template <bool... B>
+    struct and_c;
+
     template <std::size_t I, class... Args>
     constexpr decltype(auto) argument(Args&&... args) noexcept;
 
@@ -177,9 +183,9 @@ namespace xt
     {
     };
 
-    /**********************
+    /***********************
      * and_ implementation *
-     **********************/
+     ***********************/
 
     template <>
     struct and_<> : std::integral_constant<bool, true>
@@ -192,6 +198,20 @@ namespace xt
     {
     };
 
+    /**********************************
+     * or_c and and_c implementations *
+     **********************************/
+
+    template <bool... B>
+    struct or_c : or_<std::integral_constant<bool, B>...>
+    {
+    };
+
+    template <bool... B>
+    struct and_c : and_<std::integral_constant<bool, B>...>
+    {
+    };
+
     /***************************
      * argument implementation *
      ***************************/
@@ -630,6 +650,36 @@ namespace xt
     template <class S>
     using const_closure_t = typename const_closure<S>::type;
 
+    /******************************
+     * ptr_closure implementation *
+     ******************************/
+
+    template <class S>
+    struct ptr_closure
+    {
+        using underlying_type = std::conditional_t<std::is_const<std::remove_reference_t<S>>::value,
+                                                   const std::decay_t<S>,
+                                                   std::decay_t<S>>;
+        using type = std::conditional_t<std::is_lvalue_reference<S>::value,
+                                        underlying_type*,
+                                        underlying_type>;
+    };
+
+    template <class S>
+    using ptr_closure_t = typename ptr_closure<S>::type;
+
+    template <class S>
+    struct const_ptr_closure
+    {
+        using underlying_type = const std::decay_t<S>;
+        using type = std::conditional_t<std::is_lvalue_reference<S>::value,
+                                        underlying_type*,
+                                        underlying_type>;
+    };
+
+    template <class S>
+    using const_ptr_closure_t = typename const_ptr_closure<S>::type;
+
     /***************************
      * apply_cv implementation *
      ***************************/
@@ -718,6 +768,25 @@ namespace xt
         static constexpr bool value = detail::is_complex<std::decay_t<T>>::value;
     };
 
+    /*************************************
+     * complex_value_type implementation *
+     *************************************/
+
+    template <typename T>
+    struct complex_value_type
+    {
+        using type = T;
+    };
+
+    template <typename T>
+    struct complex_value_type<std::complex<T>>
+    {
+        using type = T;
+    };
+
+    template <class T>
+    using complex_value_type_t = typename complex_value_type<T>::type;
+
     /*********************************
      * forward_offset implementation *
      *********************************/
